#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>

#include "ahx/AHX.h"
#include "ahx/Windows.h"
#include "Debug.h"
#include "Main.h"
#include "resource.h"
 
void AHXInfoDialog::Init(HWND hParent)
{
	char Ini[256];
	int x, y, r, w;

	hInfoDlg = CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SONGINFO), hParent, (DLGPROC)_InfoDlgProc, (LPARAM)this);

	GetModuleFileName(NULL, Ini, 256); strcpy(&Ini[strlen(Ini)-3], "ini");
	bVisible = GetPrivateProfileInt("Info Window", "Visible", 0, Ini);
	x = GetPrivateProfileInt("Info Window", "left", 0, Ini);
	y = GetPrivateProfileInt("Info Window", "top", 0, Ini);
	r = GetPrivateProfileInt("Info Window", "right", 250, Ini) - x;
	w = GetPrivateProfileInt("Info Window", "bottom", 195, Ini) - y;
	SetWindowPos(hInfoDlg, NULL, x, y, r, w, SWP_NOZORDER|SWP_NOACTIVATE);
	ShowWindow(hInfoDlg, bVisible?SW_NORMAL:SW_HIDE);
}

void AHXInfoDialog::Show()
{
	bVisible = TRUE; ShowWindow(hInfoDlg, SW_NORMAL);
}

long AHXInfoDialog::InfoDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HIMAGELIST hImageList;
	
	switch(message) {
		case WM_INITDIALOG: {
			hInfoDlg = hDlg;
			hImageList = ImageList_LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_IMAGELIST), 17, 999, RGB(255, 0, 0));
			TreeView_SetImageList(GetDlgItem(hDlg, IDC_TREE), (LPARAM)hImageList, TVSIL_NORMAL);

			Unload();			
		} break;
		case WM_SIZE: {
			RECT r;

			GetClientRect(hDlg, &r);
			SetWindowPos(GetDlgItem(hDlg, IDC_TREE), NULL, 0, 0, r.right, r.bottom, SWP_NOMOVE|SWP_NOZORDER);
		} break;
		case WM_CLOSE:
			ShowWindow(hDlg, SW_HIDE); bVisible = FALSE;
			break;
	}
	return FALSE;
}

void AHXInfoDialog::Load(AHXPlayer* Player)
{
	HWND Tree = GetDlgItem(hInfoDlg, IDC_TREE);
	char temp[256];

	TreeView_DeleteAllItems(Tree);

	TV_INSERTSTRUCT is = { TVI_ROOT, TVI_LAST, 
		{
			TVIF_IMAGE | TVIF_STATE | TVIF_SELECTEDIMAGE | TVIF_TEXT,
			NULL,
			TVIS_BOLD|TVIS_EXPANDED,
			TVIS_BOLD|TVIS_EXPANDED,
			"song name",
			0,
			1,
			2,
			0
		}
	};

	sprintf(temp, "%s (AHX%d)", Player->Song.Name, Player->Song.Revision);
	is.item.pszText = temp;

	is.hParent = TreeView_InsertItem(Tree, (LPARAM)&is);
	is.item.state = is.item.stateMask = 0;

	sprintf(temp, "%dx-Speed, %d Subsong%s", Player->Song.SpeedMultiplier, Player->Song.SubsongNr, Player->Song.SubsongNr!=1?"s":"");
	is.item.iImage = is.item.iSelectedImage = 5;
	TreeView_InsertItem(Tree, (LPARAM)&is);
	
	sprintf(temp, "%d Position%s, %d Track%s", Player->Song.PositionNr, Player->Song.PositionNr!=1?"s":"",
		                                       Player->Song.TrackNr, Player->Song.TrackNr!=1?"s":"");
	is.item.iImage = is.item.iSelectedImage = 5;
	TreeView_InsertItem(Tree, (LPARAM)&is);

	is.item.iImage = 1; is.item.iSelectedImage = 2;

	sprintf(temp, "%d Instrument%s", Player->Song.InstrumentNr, Player->Song.InstrumentNr!=1?"s":"");
	is.item.state = is.item.stateMask = TVIS_EXPANDED;
	is.hParent = TreeView_InsertItem(Tree, (LPARAM)&is);

	is.item.state = is.item.stateMask = 0;
	for(int i = 1; i < Player->Song.InstrumentNr+1; i++) {
		is.item.iImage = is.item.iSelectedImage = 6;
		sprintf(temp, "%02d: %s", i, Player->Song.Instruments[i].Name);
		TreeView_InsertItem(Tree, (LPARAM)&is);
	}

	EnableWindow(Tree, TRUE);
}

void AHXInfoDialog::Unload()
{
	HWND Tree = GetDlgItem(hInfoDlg, IDC_TREE);

	TreeView_DeleteAllItems(Tree);

	TV_INSERTSTRUCT is = { TVI_ROOT, TVI_LAST, 
		{
			TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT,
			NULL,
			0,
			0,
			"<no song loaded>",
			0,
			4,
			4,
			0
		}
	};
	TreeView_InsertItem(Tree, (LPARAM)&is);
	EnableWindow(Tree, FALSE);
}

long FAR PASCAL AHXInfoDialog::_InfoDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static AHXInfoDialog* This = NULL;
	if(message == WM_INITDIALOG) This = (AHXInfoDialog*)lParam;
	return This->InfoDlgProc(hDlg, message, wParam, lParam);
}
